#!/bin/bash
# author           : P.R. faasse (faasse@nlr.nl) <prf>
# maintainer       : P.R. Faasse
# source location  : www.kde.org
# additional files : none 
# depends upon     : Qt, arts, kdebase, kdelibs
# NOTE!!: kdebinding is severly allergic to multi-threaded 'make's
# NOTE!!: running make -j2 in the script will not work...
# 25-aug-2008: version = 3.5.10 build = 64.1
# 05-nov-2008: rebuild (54.2) to accomodate the new perl version
#              additionally: a patch 'stolen' from ubuntu: something
#              seems to have been changed in the ruby interface too...
#              Some 'const char *' instead of 'char *' that causes
#              trouble when building anew. The patch was downloaded
#              from: 
#
#  wget -c http://patches.ubuntu.com/k/kdebindings/extracted/032_ruby_fix_const.diff
#
#		as usual, i've 'swallowed' it into the buildfile: In my
#		experience, patches are a bit too 'volatile' on the
#		internet to rely on them to be present much later on.
#		NB: i've also edited the 3.5.9 --> 3.5.10 version in the
#		patch.
#             

#
# what package is going to be built
#
softversion='3.5.10'
softname='kdebindings'

#
# a lonesome hook
#
(

echo "-- start -----"

#
# where is the source of the thing
#
srcurl="ftp://ftp.kde.org/pub/kde/stable/$softname/src/$softname-$softversion.tar.bz2"

#
# ZenWalk standard: CPU type and architecture, as well as the
# version number of ZenWalk this package is made for (on?).
#
# arch = 'i686' -> the compiler uses only 80486 instructions
# tune = 'i686' -> the optimising is done for a PII and above CPU
#
# packageversion='46.1' means: 
# 46 -> ZenWalk-4.6 
# .  -> separator
# 1  -> first build for this zenwalk-version  
#
arch='i686'
cpu='i686'
packageversion='64.2'

#
# an extra parameter: The set of packages is part of the build of
# kde-3.5.0. I find little merit in makeing a whole load of diretories
# in /usr/src (destination) for each individual package. I'll put the
# lot of them in /usr/src/kde-3.5.0 instead
#
kdeversion="3.5.10"

#
# the package 'destination'
#
prefix=/usr

#
# parameters needed for the build process
#
buildir=$(pwd)
package="$softname-$softversion-$arch-$packageversion"

echo "-- build-time dependencies check -----"

depsok=passed

cd /var/log/packages
 
#
# dependencies check; without-python 
#
for file in \
  qt-base \
  qt-devel \
  arts \
  kdelibs \
  kdebase \
  ruby \
  jdk \
; do

   #
   # Check the dependency
   #
   if [ -f $file-* ] ; then 
      echo "package: $file is installed: `ls -d $file*`: good ...";
   else
      echo "package: $file not found: please install a $file package first";
      depsok=fail
   fi
   #
done

cd $buildir
 
echo "Dependencies check result: $depsok"
#
# check if all dependencies are there
#
if [[ $depsok == passed ]]; then
   unset $depsok;
else
   exit 1;
fi

#
# do package-specific settings 
#
# This avoids compiling a version number into KDE's .la files:
QTDIR=/usr/lib/qt
export QTDIR
source /etc/profile.d/qt.sh
#
# KDE additions:
#
KDEDIR=$prefix
PATH=$PATH:$KDEDIR/bin
MANPATH=$MANPATH:$KDEDIR/man
export KDEDIR PATH MANPATH

#
# make sure that the actually installed qt makespec is used
#

#
# prepare the build result directory
#
dest="$buildir/$package"

rm -rf $dest
#
mkdir -p $dest
mkdir -p $dest/install
mkdir -p $dest/$prefix/bin

#
# the /usr/src part
#
destsrc="$dest/usr/src/kde-$kdeversion"
mkdir -p $destsrc

#
# put slack-desc in place. Either a supplied one, or some default
# text.
#
bbstring=`echo $srcurl | sed "s/./ /g"`
cat <<EOF > $dest/install/slack-desc
$bbstring|-----handy-ruler------------------------------------------------------|
$softname: Bindings for other programming languages for kde
$softname: 
$softname: The extra stuff you'll need if you want to make kde programs  
$softname: using programming languages other than C++
$softname: 
$softname: 
$softname: Python bindings are *not* included in this build!
$softname: 
$softname: Check out pykde to install the python bindings.
$softname:
$softname: 
EOF
#
# loop over expanded package source files
#
echo "-- unpacking the source archive -----"

cd $buildir
file=$softname-$softversion.tar.bz2
srcdir="$(basename $file .tar.bz2)" 
rm -rf $srcdir
tar xf $file
cd $srcdir  

#
# set target permissions
#
chown -R root.root . 
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;

#
# patching
#
echo "-- patching ruby interface ( http://patches.ubuntu.com/k/kdebindings/extracted/032_ruby_fix_const.diff ) -----"
patch -p1 <<"EOF"
--- kdebindings-3.5.10.orig/qtruby/rubylib/qtruby/Qt.cpp	2008-06-07 00:48:29.000000000 +0000
+++ kdebindings-3.5.10/qtruby/rubylib/qtruby/Qt.cpp	2008-06-07 00:50:23.000000000 +0000
@@ -950,7 +950,7 @@
 const char *
 get_VALUEtype(VALUE ruby_value)
 {
-	char * classname = rb_obj_classname(ruby_value);
+	const char * classname = rb_obj_classname(ruby_value);
     const char *r = "";
     if(ruby_value == Qnil)
 	r = "u";
@@ -1395,7 +1395,7 @@
 }
 
 static QCString *
-find_cached_selector(int argc, VALUE * argv, VALUE klass, char * methodName)
+find_cached_selector(int argc, VALUE * argv, VALUE klass, const char * methodName)
 {
     // Look in the cache
 static QCString * mcid = 0;
@@ -1432,7 +1432,7 @@
 static VALUE
 method_missing(int argc, VALUE * argv, VALUE self)
 {
-	char * methodName = rb_id2name(SYM2ID(argv[0]));
+	const char * methodName = rb_id2name(SYM2ID(argv[0]));
     VALUE klass = rb_funcall(self, rb_intern("class"), 0);
 
 	// Look for 'thing?' methods, and try to match isThing() or hasThing() in the Smoke runtime
@@ -1482,7 +1482,7 @@
 
 			VALUE retval = rb_funcall2(qt_internal_module, rb_intern("do_method_missing"), argc+3, temp_stack);
 			if (_current_method == -1) {
-				char * op = rb_id2name(SYM2ID(argv[0]));
+				const char * op = rb_id2name(SYM2ID(argv[0]));
 				if (	qstrcmp(op, "-") == 0
 						|| qstrcmp(op, "+") == 0
 						|| qstrcmp(op, "/") == 0
@@ -1557,7 +1557,7 @@
 class_method_missing(int argc, VALUE * argv, VALUE klass)
 {
 	VALUE result = Qnil;
-	char * methodName = rb_id2name(SYM2ID(argv[0]));
+	const char * methodName = rb_id2name(SYM2ID(argv[0]));
 	VALUE * temp_stack = (VALUE *) calloc(argc+3, sizeof(VALUE));
     temp_stack[0] = rb_str_new2("Qt");
     temp_stack[1] = rb_str_new2(methodName);
@@ -1772,7 +1772,7 @@
 VALUE
 getmetainfo(VALUE self, int &offset, int &index)
 {
-    char * signalname = rb_id2name(rb_frame_last_func());
+    const char * signalname = rb_id2name(rb_frame_last_func());
     VALUE metaObject_value = rb_funcall(qt_internal_module, rb_intern("getMetaObject"), 1, self);
 
     smokeruby_object *ometa = value_obj_info(metaObject_value);
EOF

#
# configure package for compilation
#
echo "-- configure -----"
#./configure --help >$CWD/configure-kdebindings

export CFLAGS="-O2 -march=$arch -mtune=$cpu"  
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"  
./configure \
  --prefix=$prefix \
  --sysconfdir=/etc\
  --localstatedir=/var \
  --mandir=/usr/man \
  --with-ssl-dir=/usr \
  --with-xinerama \
  --disable-debug \
  --disable-libfam \
  --enable-dnotify \
  --program-prefix="" \
  --program-suffix=""  \
  --disable-dependency-tracking \
  --enable-final \
  --enable-gcc-hidden-visibility \
  DO_NOT_COMPILE="python" || exit 1

#
# make package
#
echo "-- make -----"

QMAKESPEC="/usr/share/qt/mkspecs/default" make || exit 1

#
# install package to the target directory
#
echo "-- make install -----"
make install DESTDIR=$dest

#
# add 'default' files
#
echo "-- adding default files -----"

mkdir -p $dest/usr/doc/$srcdir
cp -a \
  AUTHORS COPYING INSTALL NEWS README TODO \
  $dest/usr/doc/$srcdir 

#
# add some package-rules files
#
echo "-- adding package-rules files -----"

cp "$buildir/build-$softname.sh" \
   "$destsrc/build-$softname.sh"
chmod a+x "$destsrc/build-$softname.sh"

cat <<EOF >"$destsrc/getsrc-$softname.sh"
#
# get-the-source script
#
wget -c $srcurl
EOF

#
# a shell-scrip is more usable when it's executable
#
chmod a+x "$destsrc/getsrc-$softname.sh"

#
# post-build processing: 
# - strip executables 
# - compress man-pages
#
echo "-- pre-packaging configuration -----"

( cd $dest
  find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  for mandir in $(find . -name man -type d) ; do
    if [ -d $mandir ]; then
      ( cd $mandir
        find . -type f -name "*.?" -exec gzip -9 {} \;
      )
    fi
  done
)

#
# real packaging work: 
#
chown -R root.root $dest/$prefix
echo "-- packaging -----"

cd $dest
makepkg -p -l y -c n $dest.txz
(
 echo "-- generating md5 file -----"
 cd $buildir
 md5sum $package.txz >$package.md5
 echo "-- generating dep file -----"
 requiredbuilder -z $package.txz >$package.dep
 echo "$srcurl" >$package.src
)

#
# the other end of the lonesome hook
#
) 2>&1 | tee "$softname-$softversion-info.txt"
